package com.perforce.jbrown.workarounds;

/*
 * This class works around P4Java job087545:  getSubgroups() returns incorrect results.
 */
import com.perforce.jbrown.utils.MapCmdUtils;

import com.perforce.p4java.Log;
import com.perforce.p4java.core.IUserGroup;
import com.perforce.p4java.exception.P4JavaException;
import com.perforce.p4java.impl.generic.core.UserGroup;
import com.perforce.p4java.impl.mapbased.MapKeys;
import com.perforce.p4java.impl.mapbased.server.Parameters;
import com.perforce.p4java.option.server.GetUserGroupsOptions;
import com.perforce.p4java.server.CmdSpec;
import com.perforce.p4java.server.IOptionsServer;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

/***
 * This code stolen from 
 * @author jbrown
 */
public class GetGroupsMapCmd {


    /**
     * getUserGroups() with bug fix to correctly report the subGroups.<br>
     * See com.perforce.p4java.server.IOptionsServer.getUserGroups(java.lang.String, com.perforce.p4java.option.server.GetUserGroupsOptions
     * @param server - valid instance of IOptionsServer
     * @param userOrGroupName - user or group name
     * @param opts - command options
     * @return
     * @throws P4JavaException 
     */
	public static List<IUserGroup> getUserGroups(IOptionsServer server, String userOrGroupName, GetUserGroupsOptions opts)
							throws P4JavaException {

        List<IUserGroup> resultsList = new ArrayList<IUserGroup>();
		
		List<Map<String, Object>> resultMaps = server.execMapCmdList(
												CmdSpec.GROUPS.toString(),
												Parameters.processParameters(
														opts, null, userOrGroupName, server),
												(Map<String,Object>)null);
		
		if (resultMaps != null) {
			UserGroup ugImpl = null;
			List<String> userList = null;
			List<String> subgroupList = null;
			for (Map<String, Object> map : resultMaps) {
				
				MapCmdUtils.handleErrorStr(server, map);
				
				// The server returns the results not as a series of rows, each row
				// representing a single group, but as a series of rows, each row representing
				// a single *user*, meaning we need to do reverse correlation. At least they come
				// back sorted by group name from the server, but even still, this makes for
				// less than optimal decoding...

				String groupName = (String) map.get(MapKeys.GROUP_LC_KEY);
				
				if (groupName != null) {
					if (ugImpl == null) {
						ugImpl = new UserGroup();
						userList = new ArrayList<String>();
						ugImpl.setUsers(userList);
						subgroupList = new ArrayList<String>();
						ugImpl.setSubgroups(subgroupList);
						ugImpl.setName(groupName);
					} else if (!ugImpl.getName().equals(groupName)) {
						resultsList.add(ugImpl);
						ugImpl = new UserGroup();
						ugImpl.setName(groupName);
						userList = new ArrayList<String>();
						ugImpl.setUsers(userList);
						subgroupList = new ArrayList<String>();  // bug workaround!
						ugImpl.setSubgroups(subgroupList);
					}
					
					try {
						String userName = (String) map.get(MapKeys.USER_LC_KEY);
						String maxScanRows = (String) map.get(MapKeys.MAXSCANROWS_LC_KEY);
						String maxLockTime = (String) map.get(MapKeys.MAXLOCKTIME_LC_KEY);
						String timeout = (String) map.get(MapKeys.TIMEOUT_LC_KEY);
						String passwordTimeout = (String) map.get(MapKeys.PASSWORD_TIMEOUT_LC_KEY);
						String maxResults = (String) map.get(MapKeys.MAXRESULTS_LC_KEY);
						String isOwner = (String) map.get(MapKeys.ISOWNER_LC_KEY);
						if ((isOwner != null) && (isOwner.equalsIgnoreCase("1"))) {
							if (ugImpl.getOwners() == null) {
								ugImpl.setOwners(new ArrayList<String>());
								ugImpl.getOwners().add(userName);
							}
						}
						String isSubGroup = (String) map.get(MapKeys.ISSUBGROUP_LC_KEY);
						if ((isSubGroup != null) && isSubGroup.equals("1")) {
							subgroupList.add(userName);
						}
						else {
							userList.add(userName);
						}
						if (maxScanRows != null) {
							ugImpl.setMaxScanRows(new Integer(maxScanRows));
						}
						if (maxLockTime != null) {
							ugImpl.setMaxLockTime(new Integer(maxLockTime));
						}
						if (timeout != null) {
							ugImpl.setTimeout(new Integer(timeout));
						}
						if (maxResults != null) {
							ugImpl.setMaxResults(new Integer(maxResults));
						}
						if (passwordTimeout != null) {
							ugImpl.setPasswordTimeout(new Integer(passwordTimeout));
						}
					} catch (Throwable thr) {
						Log.warn("Unexpected exception in ServerImpl.getUserGroups: "
									+ thr.getMessage());
						Log.exception(thr);
					}
				}
			}
			
			if (ugImpl != null) {
				resultsList.add(ugImpl);
			}
		}
		
		return resultsList;
	}
    
}
